package com.github.snowdream.android.widget;

import android.content.ContentResolver;
import android.content.ContentUris;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.provider.ContactsContract;

import java.io.IOException;
import java.io.InputStream;

public class ContactImage extends SmartImage {
    private long contactId;

    public ContactImage(long contactId) {
        this.contactId = contactId;
    }

    public ContactImage(long contactId,int width, int height) {
        super(width,height);
        this.contactId = contactId;
    }

    public Bitmap getBitmap(Context context) {
        super.getBitmap(context);

        // Try getting bitmap from cache first
        String key = contactId + "";
        Bitmap bitmap = null;
        if(key != null) {
            bitmap = smartImageCache.get(key);
            if(bitmap == null) {
                bitmap = getBitmapFromContactId(context,contactId, width, height);
                if(bitmap != null){
                    smartImageCache.put(key, bitmap);
                }
            }
        }

        return bitmap;
    }

    @SuppressWarnings("unused")
    private Bitmap getBitmapFromContactId(Context context,long contactId){
        Bitmap bitmap = null;

        ContentResolver contentResolver = context.getContentResolver();

        try {
            Uri uri = ContentUris.withAppendedId(ContactsContract.Contacts.CONTENT_URI, contactId);
            InputStream input = ContactsContract.Contacts.openContactPhotoInputStream(contentResolver, uri);
            bitmap = BitmapFactory.decodeStream(input);
            //bitmap = getBitmapFromInputStream(input, width, height);
        } catch(Exception e) {
            e.printStackTrace();
        }

        return bitmap;
    }

    private Bitmap getBitmapFromContactId(Context context,long contactId, int width, int height){
        Bitmap bitmap = null;
        InputStream is = null;
        BitmapFactory.Options opts = null;

        ContentResolver contentResolver = context.getContentResolver();
        Uri uri = ContentUris.withAppendedId(ContactsContract.Contacts.CONTENT_URI, contactId);


        try {
            if (width > 0 && height > 0) {  
                is = ContactsContract.Contacts.openContactPhotoInputStream(contentResolver, uri);

                opts = new BitmapFactory.Options();
                opts.inJustDecodeBounds = true;
                BitmapFactory.decodeStream(is, null, opts);

                // 计算图片缩放比例
                final int minSideLength = Math.min(width, height);
                opts.inSampleSize = BitmapUtils.computeSampleSize(opts.outWidth, opts.outHeight, minSideLength,
                        width * height);
                opts.inJustDecodeBounds = false;
                opts.inInputShareable = true;
                opts.inPurgeable = true;

                if (is != null) {
                    try {
                        is.close();
                        is = null;
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }

            is = ContactsContract.Contacts.openContactPhotoInputStream(contentResolver, uri);

            bitmap =  BitmapFactory.decodeStream(is, null, opts);
            //bitmap = getBitmapFromInputStream(input, width, height);
        } catch(Exception e) {
            e.printStackTrace();
        }finally{
            if (is != null) {
                try {
                    is.close();
                    is = null;
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }

        return bitmap;
    }
    
    public void cancel(){
        super.cancel();
        
        /*if(smartImageCache != null) {
            String key = contactId + "";
            smartImageCache.cancel(key);
        }*/
    }
}