package com.github.snowdream.android.widget;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.text.TextUtils;

public class LocalImage extends SmartImage {

    private String pathName = "";

    public LocalImage(String pathName) {
        this.pathName = pathName;
    }

    public LocalImage(String pathName,int width, int height) {
        super(width,height);
        this.pathName = pathName;
    }

    public Bitmap getBitmap(Context context) {
        super.getBitmap(context);

        // Try getting bitmap from cache first
        Bitmap bitmap = null;
        if(!TextUtils.isEmpty(pathName)) {
            bitmap = smartImageCache.get(pathName);
            if(bitmap == null) {
                bitmap = getBitmapFromPath(pathName,width,height);
                if(bitmap != null){
                    smartImageCache.put(pathName, bitmap);
                }
            }
        }

        return bitmap;
    }

    @SuppressWarnings("unused")
    private Bitmap getBitmapFromPath(String pathName) {
        Bitmap bitmap = null;
        BitmapFactory.Options opts = null;
        opts = new BitmapFactory.Options();
        opts.inInputShareable = true;
        opts.inPurgeable = true;
        
        if (pathName.contains("content")){
            Uri uri = Uri.parse(pathName);
            pathName = uri.getPath();
        }
        
        try {
            bitmap = BitmapFactory.decodeFile(pathName);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return bitmap;
    }

    private Bitmap getBitmapFromPath(String pathName, int width, int height){
        Bitmap bitmap = null;
        BitmapFactory.Options opts = null;
        
        if (pathName.contains("content")){
            Uri uri = Uri.parse(pathName);
            pathName = uri.getPath();
        }
        
        try {
            if (width > 0 && height > 0) {            
                opts = new BitmapFactory.Options();
                opts.inJustDecodeBounds = true;
                BitmapFactory.decodeFile(pathName,opts);

                // 计算图片缩放比例
                final int minSideLength = Math.min(width, height);
                opts.inSampleSize = BitmapUtils.computeSampleSize(opts.outWidth, opts.outHeight, minSideLength,
                        width * height);
                opts.inJustDecodeBounds = false;
                opts.inInputShareable = true;
                opts.inPurgeable = true;
            }

            bitmap =  BitmapFactory.decodeFile(pathName,opts);
        } catch(Exception e) {
            e.printStackTrace();
        }
        
        return bitmap;
    }
    
    public void cancel(){
        super.cancel();
        
        /*if(smartImageCache != null) {
            String key = resId + "";
            smartImageCache.cancel(key);
        }*/
    }
}
