package com.github.snowdream.android.widget;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;

import java.io.IOException;
import java.io.InputStream;

public class RawImage extends SmartImage {

    private Integer resId = 0;

    public RawImage(Integer resId) {
        this.resId = resId;
    }

    public RawImage(Integer resId,int width, int height) {
        super(width,height);
        this.resId = resId;
    }

    public Bitmap getBitmap(Context context) {
        super.getBitmap(context);

        // Try getting bitmap from cache first
        Bitmap bitmap = null;
        if(resId != null && resId > 0) {
            bitmap = smartImageCache.get(resId + "");
            if(bitmap == null) {
                bitmap = getBitmapFromRawId(context,resId,width,height);
                if(bitmap != null){
                    smartImageCache.put(resId + "", bitmap);
                }
            }
        }

        return bitmap;
    }

    @SuppressWarnings("unused")
    private Bitmap getBitmapFromRawId(Context context,Integer resId) {
        Bitmap bitmap = null;

        try {
            InputStream is = context.getResources().openRawResource(resId);

            // bitmap = getBitmapFromInputStream(is, width, height);
            bitmap = BitmapFactory.decodeStream(is);
            //bitmap = BitmapFactory.decodeResource(context.getResources(), resId);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return bitmap;
    }

    private Bitmap getBitmapFromRawId(Context context,Integer resId, int width, int height){
        Bitmap bitmap = null;
        InputStream is = null;
        BitmapFactory.Options opts = null;

        try {
            if (width > 0 && height > 0) {            
                is = context.getResources().openRawResource(resId);

                opts = new BitmapFactory.Options();
                opts.inJustDecodeBounds = true;
                BitmapFactory.decodeStream(is, null, opts);

                // 计算图片缩放比例
                final int minSideLength = Math.min(width, height);
                opts.inSampleSize = BitmapUtils.computeSampleSize(opts.outWidth, opts.outHeight, minSideLength,
                        width * height);
                opts.inJustDecodeBounds = false;
                opts.inInputShareable = true;
                opts.inPurgeable = true;

                if (is != null) {
                    try {
                        is.close();
                        is = null;
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }

            is = context.getResources().openRawResource(resId);

            bitmap =  BitmapFactory.decodeStream(is, null, opts);
            //bitmap = getBitmapFromInputStream(input, width, height);
        } catch(Exception e) {
            e.printStackTrace();
        }finally{
            if (is != null) {
                try {
                    is.close();
                    is = null;
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return bitmap;
    }
    
    public void cancel(){
        super.cancel();
        
        /*if(smartImageCache != null) {
            String key = resId + "";
            smartImageCache.cancel(key);
        }*/
    }
}
