package com.github.snowdream.android.widget;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Rect;
import android.util.AttributeSet;
import android.widget.ImageView;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class SmartImageView extends ImageView {
    private static final int LOADING_THREADS = 5;
    private static ExecutorService threadPool = Executors.newFixedThreadPool(LOADING_THREADS);
     //private static ExecutorService threadPool = ExecutorDelegate.getExecutorService();

    private SmartImageTask currentTask = null;

    public SmartImageView(Context context) {
        super(context);
    }

    public SmartImageView(Context context, AttributeSet attrs) {
        super(context, attrs);
    }

    public SmartImageView(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
    }

    // Helpers to set image by URL with width and height
    public void setImageUrl(String url,Rect rect) {
        if (rect != null) {
            setImage(new WebImage(url,rect.width(),rect.height()));
        }else {
            setImage(new WebImage(url));
        }
    }

    public void setImageUrl(String url,Rect rect, SmartImageTask.OnCompleteListener completeListener) {
        if (rect != null) {
            setImage(new WebImage(url,rect.width(),rect.height()), completeListener);
        }else {
            setImage(new WebImage(url), completeListener);
        }

    }

    public void setImageUrl(String url,Rect rect, final Integer fallbackResource) {
        if (rect != null) {
            setImage(new WebImage(url,rect.width(),rect.height()), fallbackResource);
        }else {
            setImage(new WebImage(url), fallbackResource);   
        }
    }

    public void setImageUrl(String url,Rect rect, final Integer fallbackResource, SmartImageTask.OnCompleteListener completeListener) {
        if (rect != null) {
            setImage(new WebImage(url,rect.width(),rect.height()), fallbackResource, completeListener);
        }else {
            setImage(new WebImage(url), fallbackResource, completeListener);
        }
    }

    public void setImageUrl(String url,Rect rect, final Integer fallbackResource, final Integer loadingResource) {
        if (rect != null) {
            setImage(new WebImage(url,rect.width(),rect.height()), fallbackResource, loadingResource);
        }else {
            setImage(new WebImage(url), fallbackResource, loadingResource);
        }
    }

    public void setImageUrl(String url,Rect rect, final Integer fallbackResource, final Integer loadingResource, SmartImageTask.OnCompleteListener completeListener) {
        if (rect != null) {
            setImage(new WebImage(url,rect.width(),rect.height()), fallbackResource, loadingResource, completeListener);
        }else {
            setImage(new WebImage(url), fallbackResource, loadingResource, completeListener);
        }
    }


    // Helpers to set image by RawId with width and height
    public void setImageRawId(Integer resId,Rect rect) {
        if (rect != null) {
            setImage(new RawImage(resId,rect.width(),rect.height()));
        }else {
            setImage(new RawImage(resId));
        }
    }

    public void setImageRawId(Integer resId,Rect rect, SmartImageTask.OnCompleteListener completeListener) {
        if (rect != null) {
            setImage(new RawImage(resId,rect.width(),rect.height()), completeListener);
        }else {
            setImage(new RawImage(resId), completeListener);
        }
    }

    public void setImageRawId(Integer resId,Rect rect, final Integer fallbackResource) {
        if (rect != null) {
            setImage(new RawImage(resId,rect.width(),rect.height()), fallbackResource);
        }else {
            setImage(new RawImage(resId), fallbackResource);
        }
    }

    public void setImageRawId(Integer resId,Rect rect, final Integer fallbackResource, SmartImageTask.OnCompleteListener completeListener) {
        if (rect != null) {
            setImage(new RawImage(resId,rect.width(),rect.height()), fallbackResource, completeListener);
        }else {
            setImage(new RawImage(resId), fallbackResource, completeListener); 
        }
    }

    public void setImageRawId(Integer resId,Rect rect, final Integer fallbackResource, final Integer loadingResource) {
        if (rect != null) {
            setImage(new RawImage(resId,rect.width(),rect.height()), fallbackResource, loadingResource);
        }else {
            setImage(new RawImage(resId), fallbackResource, loadingResource);
        }
    }

    public void setImageRawId(Integer resId,Rect rect, final Integer fallbackResource, final Integer loadingResource, SmartImageTask.OnCompleteListener completeListener) {
        if (rect != null) {
            setImage(new RawImage(resId,rect.width(),rect.height()), fallbackResource, loadingResource, completeListener);
        }else {
            setImage(new RawImage(resId), fallbackResource, loadingResource, completeListener);
        }
    }
    
    // Helpers to set image by image path with width and height
    public void setImagePath(String pathName,Rect rect) {
        if (rect != null) {
            setImage(new LocalImage(pathName,rect.width(),rect.height()));
        }else {
            setImage(new LocalImage(pathName));
        }
    }

    public void setImagePath(String pathName,Rect rect, SmartImageTask.OnCompleteListener completeListener) {
        if (rect != null) {
            setImage(new LocalImage(pathName,rect.width(),rect.height()), completeListener);
        }else {
            setImage(new LocalImage(pathName), completeListener);
        }
    }

    public void setImagePath(String pathName,Rect rect, final Integer fallbackResource) {
        if (rect != null) {
            setImage(new LocalImage(pathName,rect.width(),rect.height()), fallbackResource);
        }else {
            setImage(new LocalImage(pathName), fallbackResource);
        }
    }

    public void setImagePath(String pathName,Rect rect, final Integer fallbackResource, SmartImageTask.OnCompleteListener completeListener) {
        if (rect != null) {
            setImage(new LocalImage(pathName,rect.width(),rect.height()), fallbackResource, completeListener);
        }else {
            setImage(new LocalImage(pathName), fallbackResource, completeListener); 
        }
    }

    public void setImagePath(String pathName,Rect rect, final Integer fallbackResource, final Integer loadingResource) {
        if (rect != null) {
            setImage(new LocalImage(pathName,rect.width(),rect.height()), fallbackResource, loadingResource);
        }else {
            setImage(new LocalImage(pathName), fallbackResource, loadingResource);
        }
    }

    public void setImagePath(String pathName,Rect rect, final Integer fallbackResource, final Integer loadingResource, SmartImageTask.OnCompleteListener completeListener) {
        if (rect != null) {
            setImage(new LocalImage(pathName,rect.width(),rect.height()), fallbackResource, loadingResource, completeListener);
        }else {
            setImage(new LocalImage(pathName), fallbackResource, loadingResource, completeListener);
        }
    }

    // Helpers to set image by Drawable Id
    public void setImageDrawableId(Integer resId,Rect rect) {
        if (rect != null) {
            setImage(new DrawableImage(resId,rect.width(),rect.height()));
        }else {
            setImage(new DrawableImage(resId)); 
        }
    }

    public void setImageDrawableId(Integer resId,Rect rect, SmartImageTask.OnCompleteListener completeListener) {
        if (rect != null) {
            setImage(new DrawableImage(resId,rect.width(),rect.height()), completeListener);
        }else {
            setImage(new DrawableImage(resId), completeListener);
        }
    }

    public void setImageDrawableId(Integer resId,Rect rect, final Integer fallbackResource) {
        if (rect != null) {
            setImage(new DrawableImage(resId,rect.width(),rect.height()), fallbackResource);
        }else {
            setImage(new DrawableImage(resId), fallbackResource);
        }
    }

    public void setImageDrawableId(Integer resId,Rect rect, final Integer fallbackResource, SmartImageTask.OnCompleteListener completeListener) {
        if (rect != null) {
            setImage(new DrawableImage(resId,rect.width(),rect.height()), fallbackResource, completeListener);
        }else {
            setImage(new DrawableImage(resId), fallbackResource, completeListener);
        }
    }

    public void setImageDrawableId(Integer resId,Rect rect, final Integer fallbackResource, final Integer loadingResource) {
        if (rect != null) {
            setImage(new DrawableImage(resId,rect.width(),rect.height()), fallbackResource, loadingResource);
        }else {
            setImage(new DrawableImage(resId), fallbackResource, loadingResource);
        }
    }

    public void setImageDrawableId(Integer resId,Rect rect, final Integer fallbackResource, final Integer loadingResource, SmartImageTask.OnCompleteListener completeListener) {
        if (rect != null) {
            setImage(new DrawableImage(resId,rect.width(),rect.height()), fallbackResource, loadingResource, completeListener);
        }else {
            setImage(new DrawableImage(resId), fallbackResource, loadingResource, completeListener);
        }
    }

    // Helpers to set image by contact address book id width and height
    public void setImageContact(long contactId,Rect rect) {
        if (rect != null) {
            setImage(new ContactImage(contactId,rect.width(),rect.height()));
        }else {
            setImage(new ContactImage(contactId));
        }
    }

    public void setImageContact(long contactId,Rect rect, final Integer fallbackResource) {
        if (rect != null) {
            setImage(new ContactImage(contactId,rect.width(),rect.height()), fallbackResource);
        }else {
            setImage(new ContactImage(contactId), fallbackResource);
        }
    }

    public void setImageContact(long contactId,Rect rect, final Integer fallbackResource, final Integer loadingResource) {
        if (rect != null) {
            setImage(new ContactImage(contactId,rect.width(),rect.height()), fallbackResource, fallbackResource);
        }else {
            setImage(new ContactImage(contactId), fallbackResource, fallbackResource);   
        }
    }

    // Set image using SmartImage object
    public void setImage(final SmartImage image) {
        setImage(image, null, null, null);
    }

    public void setImage(final SmartImage image, final SmartImageTask.OnCompleteListener completeListener) {
        setImage(image, null, null, completeListener);
    }

    public void setImage(final SmartImage image, final Integer fallbackResource) {
        setImage(image, fallbackResource, fallbackResource, null);
    }

    public void setImage(final SmartImage image, final Integer fallbackResource, SmartImageTask.OnCompleteListener completeListener) {
        setImage(image, fallbackResource, fallbackResource, completeListener);
    }

    public void setImage(final SmartImage image, final Integer fallbackResource, final Integer loadingResource) {
        setImage(image, fallbackResource, loadingResource, null);
    }

    public void setImage(final SmartImage image, final Integer fallbackResource, final Integer loadingResource, final SmartImageTask.OnCompleteListener completeListener) {
        // Set a loading resource
        if(loadingResource != null){
            setImageResource(loadingResource);
        }

        // Cancel any existing tasks for this image view
        if(currentTask != null) {
            currentTask.cancel();
            currentTask = null;
        }

        // Set up the new task
        currentTask = new SmartImageTask(getContext(), image);
        currentTask.setOnCompleteHandler(new SmartImageTask.OnCompleteHandler() {
            @Override
            public void onComplete(Bitmap bitmap) {
                if(bitmap != null) {
                    setImageBitmap(bitmap);
                } else {
                    // Set fallback resource
                    if(fallbackResource != null) {
                        setImageResource(fallbackResource);
                    }
                }

                if(completeListener != null){
                    completeListener.onComplete();
                }
            }
        });

        // Run the task in a threadpool
        threadPool.execute(currentTask);
    }
    
    public void cancel() {
        // Cancel any existing tasks for this image view
        if(currentTask != null) {
            currentTask.cancel();
            currentTask = null;
        }
    }
    
    public static void cancelAllTasks() {
        threadPool.shutdownNow();
        //threadPool = ExecutorDelegate.getExecutorService();
        threadPool = Executors.newFixedThreadPool(LOADING_THREADS);
    }
}