package com.github.snowdream.android.widget;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;

import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;

public class WebImage extends SmartImage {
    private static final int CONNECT_TIMEOUT = 5000;
    private static final int READ_TIMEOUT = 10000;

    private String url;

    public WebImage(String url) {
        this.url = url;
    }

    public WebImage(String url,int width, int height) {
        super(width,height);
        this.url = url;
    }

    public Bitmap getBitmap(Context context) {
        super.getBitmap(context);

        // Try getting bitmap from cache first
        Bitmap bitmap = null;
        if(url != null) {

            if (width == 0 | height == 0) {
                bitmap = smartImageCache.get(url);
                if(bitmap == null) {
                    bitmap = getBitmapFromUrl(url);
                    if(bitmap != null){
                        smartImageCache.put(url, bitmap);
                    }
                }
            }else {
                bitmap = smartImageCache.get(url,width,height);
                if(bitmap == null) {
                    bitmap = getBitmapFromUrl(url,width,height);
                    if(bitmap != null){
                        //smartImageCache.put(url, bitmap);
                    }
                }
            }

        }

        return bitmap;
    }

    private Bitmap getBitmapFromUrl(String url) {
        Bitmap bitmap = null;

        try {
            URL imagePath = new URL(url);
            HttpURLConnection conn = (HttpURLConnection)imagePath.openConnection();
            conn.setConnectTimeout(CONNECT_TIMEOUT);
            conn.setReadTimeout(READ_TIMEOUT);
            conn.connect();
            InputStream is = (InputStream) conn.getContent();
            // bitmap = getBitmapFromInputStream(is, width, height);
            bitmap = BitmapFactory.decodeStream(is);
        } catch(Exception e) {
            e.printStackTrace();
        }

        /*String[] allowedContentTypes = new String[] { "image/png", "image/jpeg" };
        FounderHttpClient.doGet("http://example.com/file.png", new BinaryHttpResponseHandler(allowedContentTypes){

            @Override
            public void onSuccess(byte[] binaryData) {
                Bitmap bitmap = null;

                bitmap = BitmapFactory.decodeByteArray(binaryData, 0, binaryData.length);
            }

        });*/

        return bitmap;
    }

    private Bitmap getBitmapFromUrl(String url,int width, int height) {
        Bitmap bitmap = null;

        try {
            URL imagePath = new URL(url);
            HttpURLConnection conn = (HttpURLConnection)imagePath.openConnection();
            conn.setConnectTimeout(CONNECT_TIMEOUT);
            conn.setReadTimeout(READ_TIMEOUT);
            conn.connect();
            InputStream is = (InputStream) conn.getContent();
            if (is != null) {
                // bitmap = getBitmapFromInputStream(is, width, height);
                // bitmap = BitmapFactory.decodeStream((InputStream) conn.getContent());
                smartImageCache.put(url, is, width, height);
                bitmap = smartImageCache.get(url, width, height);
            }
        } catch(Exception e) {
            e.printStackTrace();
        }

        /*InputStream is =  FounderSyncHttpClient.doGet(url);
        if (is != null) {
            // bitmap = getBitmapFromInputStream(is, width, height);
            // bitmap = BitmapFactory.decodeStream((InputStream) conn.getContent());
            smartImageCache.put(url, is, width, height);
            bitmap = smartImageCache.get(url, width, height);
        }*/

        return bitmap;
    }
    
    public void cancel(){
        super.cancel();
        
        if(smartImageCache != null) {
            smartImageCache.cancel(url);
        }
    }
}